@@ -382,9 +382,9 @@ class KernelDG(nx.DiGraph):
             # determine absolute address change
             addr_change = 0
             if src.offset and "value" in src.offset:
-                addr_change += int(src.offset.value, 0)
+                addr_change += src.offset.value
             if mem.offset:
-                addr_change -= int(mem.offset.value, 0)
+                addr_change -= mem.offset.value
             if mem.base and src.base:
                 base_change = register_changes.get(
                     src.base.get('prefix', '') + src.base.name,
